import * as jwt from 'jsonwebtoken'
import * as E from 'fp-ts/Either'
import * as O from 'fp-ts/Option'
/**
 * @since 1.0.0
 */
export interface Decode {
  readonly value: string
}
/**
 * @since 1.0.0
 */
export interface Encode<DATA = unknown> {
  readonly value: {
    readonly data: DATA
  }
}
/**
 * @since 1.0.0
 */
export declare const ERROR: {
  readonly NO_SECOND_ELEMENT: 'NO_SECOND_ELEMENT'
  readonly NULL_OR_UNDEFINED: 'NULL_OR_UNDEFINED'
}
/**
 * @since 1.0.0
 */
export declare type CustomErrorsMessage = typeof ERROR[keyof typeof ERROR]
/**
 * @since 1.0.0
 */
export declare const Jwt: {
  decode: <JWT>({ value }: Decode) => E.Either<
    SyntaxError | CustomErrorsMessage,
    {
      readonly data: JWT
    }
  >
  encode: (
    value: Encode<unknown> & {
      readonly secretOrPrivateKey: jwt.Secret
      readonly options: O.Option<jwt.SignOptions>
    }
  ) => E.Either<Error | jwt.JsonWebTokenError, string>
}
