/**
 * @since 1.0.0
 * @description
 *  JWT Encode and Decode
 *
 */
import { pipe, flow } from 'fp-ts/function';
import * as jwt from 'jsonwebtoken';
import * as E from 'fp-ts/Either';
import * as RA from 'fp-ts/ReadonlyArray';
import * as J from 'fp-ts/Json';
import * as O from 'fp-ts/Option';
/**
 * @since 1.0.0
 */
export var ERROR = {
    NO_SECOND_ELEMENT: 'NO_SECOND_ELEMENT',
    NULL_OR_UNDEFINED: 'NULL_OR_UNDEFINED'
};
var splitByDots = function (s) { return E.right(s.split('.')); };
var BufferFromb64Encoded = function (b64Encoded) { return Buffer.from(b64Encoded, 'base64').toString(); };
var fromBuffer = function (e) { return E.right(BufferFromb64Encoded(e)); };
var takeSecondElement = function (ra) {
    return pipe(RA.lookup(1, ra), E.fromOption(function () { return new Error(ERROR.NO_SECOND_ELEMENT); }));
};
var replaceDashWithPlus = function (c) { return c.replace(/-/g, '+'); };
var replaceUnderscoreWithSlash = function (c) { return c.replace(/_/g, '/'); };
/**
 * @description
 * Decode a JWT into a string
 * @since 1.0.0
 */
var decode = function (_a) {
    var value = _a.value;
    return pipe(value, E.fromNullable(new Error(ERROR.NULL_OR_UNDEFINED)), E.chain(splitByDots), E.chain(takeSecondElement), E.map(flow(replaceDashWithPlus, replaceUnderscoreWithSlash)), E.chain(fromBuffer), E.chain(J.parse));
};
var sign = function (payload, secretOrPrivateKey, maybeOptions) {
    return pipe(maybeOptions, O.map(function (options) { return jwt.sign(payload, secretOrPrivateKey, options); }), O.getOrElse(function () { return jwt.sign(payload, secretOrPrivateKey); }));
};
/**
 * @description
 * Encode a string in a JWT
 * @since 1.0.0
 */
var encode = function (_a) {
    var value = _a.value, secretOrPrivateKey = _a.secretOrPrivateKey, options = _a.options;
    return pipe(value, E.fromNullable(new Error(ERROR.NULL_OR_UNDEFINED)), E.map(function (x) { return sign(x, secretOrPrivateKey, options); }));
};
/**
 * @since 1.0.0
 */
export var Jwt = {
    decode: decode,
    encode: encode
};
